/* $Id: cpl_fft-test.c,v 1.2 2009/12/14 15:35:53 llundin Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: llundin $
 * $Date: 2009/12/14 15:35:53 $
 * $Revision: 1.2 $
 * $Name:  $
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

/*-----------------------------------------------------------------------------
                                Includes
 -----------------------------------------------------------------------------*/
#include <float.h>
#include <complex.h>

#include "cpl_memory.h"
#include "cpl_stats.h"
#include "cpl_tools.h"
#include "cpl_test.h"
#include "cpl_math_const.h"

#include "cpl_image_io_impl.h"

#include "cpl_fft.h"

/*-----------------------------------------------------------------------------
                                Defines
 -----------------------------------------------------------------------------*/

#ifndef IMAGESZ
#define IMAGESZ         10
#endif

#ifndef CONSTANT
#define CONSTANT        200
#endif

/*----------------------------------------------------------------------------*/
/**
 * @defgroup cpl_fit_test Testing of the CPL utilities
 */
/*----------------------------------------------------------------------------*/



/*-----------------------------------------------------------------------------
                            Private Function prototypes
 -----------------------------------------------------------------------------*/
static void cpl_fft_image_tests(void);
//static void cpl_fit_imagelist_tests(void);

/*----------------------------------------------------------------------------*/
/**
   @brief   Unit tests of fit module
**/
/*----------------------------------------------------------------------------*/

int main(void)
{

    cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);

    /* Insert tests below */

    cpl_fft_image_tests();
//    cpl_fit_imagelist_tests();

    /* End of tests */
    return cpl_test_end(0);
}


static void cpl_fft_image_tests(void)
{
#ifdef CPL_FFTW_INSTALLED 
#ifdef CPL_FFTWF_INSTALLED 

    const cpl_type     imtypes[] = {CPL_TYPE_DOUBLE, CPL_TYPE_FLOAT,
                                    CPL_TYPE_INT, CPL_TYPE_DOUBLE_COMPLEX,
                    CPL_TYPE_FLOAT_COMPLEX};

    int ityp;

    /* Insert tests below */

    /* Iterate through all pixel types */
    for (ityp = 0; ityp < (int)(sizeof(imtypes)/sizeof(imtypes[0])); ityp++) {
        const cpl_type imtype = imtypes[ityp];

        int ityp2;

        cpl_image * img1 = cpl_image_new(IMAGESZ, IMAGESZ, imtype);

    if (imtype & CPL_TYPE_COMPLEX) 
        if (imtype & CPL_TYPE_FLOAT) {
        float complex * data = cpl_image_get_data_float_complex(img1);
        int i;

        for (i = 0; i < IMAGESZ * IMAGESZ; i++) {
            data[i] = (float complex) (CONSTANT + CONSTANT * I);
        }
        } else {
        double complex * data = cpl_image_get_data_double_complex(img1);
        int i;

        for (i = 0; i < IMAGESZ * IMAGESZ; i++) {
            data[i] = (double complex) (CONSTANT + CONSTANT * I);
        }
        }
    else
        cpl_image_fill_noise_uniform(img1, 0, CONSTANT);

    for (ityp2 = 0; ityp2 < (int)(sizeof(imtypes)/sizeof(imtypes[0]));
         ityp2++) {
        const cpl_type imtype2 = imtypes[ityp2];
        
        cpl_error_code error1, error2, error3, error4, error5, error6;

        cpl_image * img2 = cpl_image_new(IMAGESZ, IMAGESZ, imtype2);
        cpl_image * img3 = cpl_image_new(IMAGESZ, IMAGESZ, imtype);
        cpl_image * img4 = cpl_image_new(IMAGESZ, IMAGESZ, imtype2);

            cpl_msg_info(cpl_func,
             "Testing with image types - input: '%s' output: '%s'",
             cpl_type_get_name(imtype),
             cpl_type_get_name(imtype2));

        error1 = cpl_fft_image(img2, img1, CPL_FFT_FORWARD);
        cpl_error_reset();
        error2 = cpl_fft_image(img2, img1, CPL_FFT_BACKWARD);
        cpl_error_reset();
        error3 = cpl_fft_image(img3, img2, CPL_FFT_FORWARD);
        cpl_error_reset();
        error4 = cpl_fft_image(img3, img2, CPL_FFT_BACKWARD);
        cpl_error_reset();
        error5 = cpl_fft_image(img4, img3, CPL_FFT_FORWARD);
        cpl_error_reset();
        error6 = cpl_fft_image(img4, img3, CPL_FFT_BACKWARD);
        cpl_error_reset();

        if (!(imtype & CPL_TYPE_COMPLEX) && (imtype2 & CPL_TYPE_COMPLEX)) {

        if (((imtype & CPL_TYPE_FLOAT)  && (imtype2 & CPL_TYPE_FLOAT))  ||
            ((imtype & CPL_TYPE_DOUBLE) && (imtype2 & CPL_TYPE_DOUBLE))){
            cpl_test_eq(error1, CPL_ERROR_NONE);
            cpl_test_eq(error4, CPL_ERROR_NONE);
        } else {
            cpl_test_eq(error1, CPL_ERROR_TYPE_MISMATCH);
            cpl_test_eq(error2, CPL_ERROR_TYPE_MISMATCH);
        }

        } else if ((imtype & CPL_TYPE_COMPLEX) && !(imtype2 & CPL_TYPE_COMPLEX)) {
        if (((imtype & CPL_TYPE_FLOAT)  && (imtype2 & CPL_TYPE_FLOAT)) ||
                    ((imtype & CPL_TYPE_DOUBLE) && (imtype2 & CPL_TYPE_DOUBLE))) {
            cpl_test_eq(error2, CPL_ERROR_NONE);
            cpl_test_eq(error3, CPL_ERROR_NONE);
        } else {
            cpl_test_eq(error2, CPL_ERROR_TYPE_MISMATCH);
        }
        } else {
        cpl_test_eq(error1, CPL_ERROR_ILLEGAL_INPUT);
        }

        if (error1 == CPL_ERROR_NONE || error2 == CPL_ERROR_NONE) {
        cpl_msg_info(cpl_func,
                 "Testing correctness for correct combinations");

        if (!(imtype & CPL_TYPE_COMPLEX)) {
            if (imtype & CPL_TYPE_FLOAT)
            cpl_test_image_abs(img1, img3,
                       10 * FLT_EPSILON * CONSTANT);
            else
            cpl_test_image_abs(img1, img3,
                       10 * DBL_EPSILON * CONSTANT);
        }

        if (error1 == CPL_ERROR_NONE) {
            cpl_image * img2_re = cpl_image_get_real(img2);
            cpl_image * img2_im = cpl_image_get_imag(img2);
            
            cpl_image * img4_re = cpl_image_get_real(img4);
            cpl_image * img4_im = cpl_image_get_imag(img4);
            
            if (imtype & CPL_TYPE_FLOAT) {
            cpl_test_image_abs(img2_re, img4_re,
                       2 * 10 * FLT_EPSILON * CONSTANT);
            cpl_test_image_abs(img2_im, img4_im,
                       2 * 10 * FLT_EPSILON * CONSTANT);
            } else {
            cpl_test_image_abs(img2_re, img4_re,
                       2 *10 * DBL_EPSILON * CONSTANT);
            cpl_test_image_abs(img2_im, img4_im,
                       2* 10 * DBL_EPSILON * CONSTANT);
            
            }
            
            cpl_image_delete(img2_re);
            cpl_image_delete(img2_im);
            cpl_image_delete(img4_re);
            cpl_image_delete(img4_im);
        }
        }

        cpl_image_delete(img2);
        cpl_image_delete(img3);
        cpl_image_delete(img4);
    }

    cpl_image_delete(img1);
    }

#endif
#endif

}


